package ly.hall.rose.healthsync;

import net.fabricmc.api.ClientModInitializer;
import net.fabricmc.fabric.api.client.event.lifecycle.v1.ClientTickEvents;
import net.fabricmc.fabric.api.client.networking.v1.ClientPlayNetworking;
import net.fabricmc.fabric.api.client.rendering.v1.HudRenderCallback;
import net.minecraft.entity.Entity;
import net.minecraft.entity.EntityType;
import net.minecraft.entity.EquipmentSlot;
import net.minecraft.entity.LivingEntity;
import net.minecraft.entity.damage.DamageSource;
import net.minecraft.entity.damage.DamageSources;
import net.minecraft.entity.damage.DamageType;
import net.minecraft.entity.damage.DamageTypes;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.item.ItemStack;
import net.minecraft.registry.RegistryKeys;
import net.minecraft.registry.entry.RegistryEntry;
import net.minecraft.text.Style;
import net.minecraft.text.Text;
import net.minecraft.text.Texts;
import net.minecraft.util.Arm;
import net.minecraft.util.math.Vec3d;
import org.jetbrains.annotations.Nullable;

import java.util.List;
import java.util.Objects;
import java.util.Optional;
import java.util.Random;
import java.util.concurrent.atomic.AtomicInteger;

public class HealthSyncClient implements ClientModInitializer {
	private Text getReasonString(LivingEntityHealReason r) {
		if (r == null)
			return Texts.join(Text.of("Generic").getWithStyle(Style.EMPTY.withItalic(true)), Text.of(""));

		return switch (r) {
			case Generic -> Texts.join(Text.of("Generic").getWithStyle(Style.EMPTY.withItalic(true)), Text.of(""));
			case NaturalRegeneration -> Text.of("Natural Regeneration");
			case RegenerationStatusEffect -> Text.of("Regeneration");
			case InstantHealthStatusEffect -> Text.of("Instant Health");
			case AbsorptionStatusEffect -> Text.of("Absorption");
			case Synchronization -> null;
		};
	}

	private String snakeToNormal(String snake) {
		String[] strings = snake.split("_");

		strings[0] = Character.toUpperCase(strings[0].charAt(0)) + strings[0].substring(1);

		return String.join(" ", strings);
	}

	private Text getReasonString(Optional<DamageSource> optSource, Text damagedUser) {
		if (optSource.isEmpty())
			return Texts.join(Text.of("Generic").getWithStyle(Style.EMPTY.withItalic(true)), Text.of(""));

		var source = optSource.get();

		String string = "death.attack." + source.getType().msgId();

		if (source.getAttacker() == null && source.getSource() == null)
			return Text.translatable(string, damagedUser);

		Text text = source.getAttacker() == null ? source.getSource().getDisplayName() : source.getAttacker().getDisplayName();
		Entity entity = source.getAttacker();
		ItemStack itemStack = entity instanceof LivingEntity
			? ((LivingEntity)entity).getMainHandStack()
			: ItemStack.EMPTY;

		if (!itemStack.isEmpty() && itemStack.hasCustomName()) {
			return Text.translatable(string + ".item", damagedUser, text, itemStack.toHoverableText());
		}

		return Text.translatable(string, damagedUser, text);


//		var s = source.getType().msgId();
//
//		if (s.endsWith(".item"))
//			s = s.substring(0, s.length()-5);
//
//		if (s.endsWith(".player"))
//			s = s.substring(0, s.length()-7);
//
//		return Text.of(snakeToNormal(s));
	}

	@Override
	public void onInitializeClient() {
		var o = new PlayerDamageOverlay();
		HudRenderCallback.EVENT.register(o);

//		var random = new Random();
//
//		AtomicInteger i = new AtomicInteger();

//		ClientTickEvents.END_CLIENT_TICK.register(client -> {
//			i.getAndIncrement();
//
//			if (i.get() >= 20) {
//				o.pushInstance(new PlayerDamageInstance(
//					client.getGameProfile(),
//					HeartType.NORMAL,
//					Text.of("Demo"),
//					random.ints(-10, 11).findFirst().getAsInt() < 0 ? -7 : 7
//				));
//				i.set(0);
//			}
//		});

		ClientPlayNetworking.registerGlobalReceiver(NetworkingConstants.HEAL_EVENT_PACKET_ID, (client, handler, buf, responseSender) -> {
			var uuid = buf.readUuid();
			var reason = buf.readOptional(r -> LivingEntityHealReason.valueOf(r.readString()));
			var deltaHealth = buf.readFloat();
			var deltaAbsorption = buf.readFloat();

			if (client.world == null)
				return;

			if (client.getNetworkHandler() == null)
				return;

			var listEntry = Objects.requireNonNull(client.getNetworkHandler()).getPlayerListEntry(uuid);

			if (listEntry == null)
				return;

			if (deltaHealth != 0) {
                o.pushInstance(new PlayerDamageInstance(
					Objects.requireNonNull(client.getNetworkHandler()).getPlayerListEntry(uuid).getProfile(),
					HeartType.NORMAL,
					getReasonString(reason.orElse(null)),
					deltaHealth
				));
			}

			if (deltaAbsorption != 0) {
                o.pushInstance(new PlayerDamageInstance(
					Objects.requireNonNull(client.getNetworkHandler()).getPlayerListEntry(uuid).getProfile(),
					HeartType.ABSORBING,
					getReasonString(reason.orElse(null)),
					deltaHealth
				));
			}
		});

		ClientPlayNetworking.registerGlobalReceiver(NetworkingConstants.DAMAGE_EVENT_PACKET_ID, (client, handler, buf, responseSender) -> {
			var uuid = buf.readUuid();
			var opt = buf.readOptional(bufx -> {
				var sourceTypeId = bufx.readInt();
				var attackerId = bufx.readInt();
				var sourceId = bufx.readInt();
				var storedPosition = buf.readOptional(bufxx -> new Vec3d(
					bufxx.readDouble(),
					bufxx.readDouble(),
					bufxx.readDouble()
				));

                assert client.world != null;
                RegistryEntry<DamageType> entry = client.world.getRegistryManager().get(RegistryKeys.DAMAGE_TYPE).getEntry(sourceTypeId).orElseThrow();

				Entity entity = attackerId == 0 ? null : client.world.getEntityById(attackerId - 1);
				Entity entity2 = sourceId == 0 ? null : client.world.getEntityById(sourceId - 1);

                return storedPosition
						.map(vec3d -> new DamageSource(entry, vec3d))
						.orElseGet(() -> new DamageSource(entry, entity, entity2));
            });
			var deltaTotal = buf.readFloat() + buf.readFloat();
			var isPoison = buf.readBoolean();
			var isWither = buf.readBoolean();
			var isFrozen = buf.readBoolean();

			if (client.world == null)
				return;

			if (client.getNetworkHandler() == null)
				return;

			var listEntry = Objects.requireNonNull(client.getNetworkHandler()).getPlayerListEntry(uuid);

			if (listEntry == null)
				return;

			if (opt.isPresent() && opt.get().isOf(DamageTypes.PLAYER_ATTACK) && opt.get().getAttacker() != null) {
				o.pushInstance(new PlayerDamageInstance(
					((PlayerEntity)opt.get().getAttacker()).getGameProfile(),
					HeartType.CONTAINER,
					Text.of("Homicide"),
					deltaTotal
				));
			}

            o.pushInstance(new PlayerDamageInstance(
				listEntry.getProfile(),
				isPoison ? HeartType.POISONED : isWither ? HeartType.WITHERED : isFrozen ? HeartType.FROZEN : HeartType.CONTAINER,
				getReasonString(opt, listEntry.getDisplayName() == null ? Text.of(listEntry.getProfile().getName()) : listEntry.getDisplayName()),
				deltaTotal
			));
		});
	}
}