package ly.hall.rose.healthsync;

import com.mojang.blaze3d.systems.RenderSystem;
import net.minecraft.client.MinecraftClient;
import net.minecraft.client.gui.DrawContext;
import net.minecraft.client.render.*;
import net.minecraft.client.util.Icons;
import net.minecraft.client.util.math.MatrixStack;
import net.minecraft.util.Identifier;
import org.joml.Matrix4f;

public enum HeartType {
    CONTAINER(0, false, true),
    NORMAL(2, true, false),
    POISONED(4, true, false),
    WITHERED(6, true, false),
    ABSORBING(8, false, false),
    FROZEN(9, false, false);

    private final int textureIndex;
    private final boolean hasBlinkingTexture;
    private final boolean shouldCrop;

    private HeartType(int textureIndex, boolean hasBlinkingTexture, boolean shouldCrop) {
        this.textureIndex = textureIndex;
        this.hasBlinkingTexture = hasBlinkingTexture;
        this.shouldCrop = shouldCrop;
    }

    public int getU(boolean halfHeart, boolean blinking) {
        int i;
        if (this == CONTAINER) {
            i = blinking ? 1 : 0;
        } else {
            int j = halfHeart ? 1 : 0;
            int k = this.hasBlinkingTexture && blinking ? 2 : 0;
            i = j + k;
        }
        return 16 + (this.textureIndex * 2 + i) * 9;
    }

    public void draw(DrawContext ctx, int x, int y, boolean hardcore, boolean half, boolean blinking, boolean last, float opacity) {
        int width = (half && this.shouldCrop) ? 5 : (last ? 9 : 8);
        int height = 9;

        var ta = new Identifier("textures/gui/icons.png");

        int x1 = x;
        int y1 = y;
        int x2 = x + width;
        int y2 = y + height;

        float u1 = this.getU(half, blinking);
        float v1 = (hardcore ? 5 : 0) * 9;
        float u2 = u1 + width;
        float v2 = v1 + height;

        RenderSystem.setShaderTexture(0, ta);
        RenderSystem.setShader(GameRenderer::getPositionColorTexProgram);
        RenderSystem.enableBlend();
        Matrix4f matrix4f = (new MatrixStack()).peek().getPositionMatrix();
        BufferBuilder bufferBuilder = Tessellator.getInstance().getBuffer();
        bufferBuilder.begin(VertexFormat.DrawMode.QUADS, VertexFormats.POSITION_COLOR_TEXTURE);
        bufferBuilder.vertex(matrix4f, x1, y1, 0).color(1.0F, 1.0F, 1.0F, opacity).texture(u1/256, v1/256).next();
        bufferBuilder.vertex(matrix4f, x1, y2, 0).color(1.0F, 1.0F, 1.0F, opacity).texture(u1/256, v2/256).next();
        bufferBuilder.vertex(matrix4f, x2, y2, 0).color(1.0F, 1.0F, 1.0F, opacity).texture(u2/256, v2/256).next();
        bufferBuilder.vertex(matrix4f, x2, y1, 0).color(1.0F, 1.0F, 1.0F, opacity).texture(u2/256, v1/256).next();
        BufferRenderer.drawWithGlobalProgram(bufferBuilder.end());
        RenderSystem.disableBlend();
    }
}