package ly.hall.rose.healthsync;

import com.mojang.authlib.minecraft.MinecraftProfileTexture;
import com.mojang.blaze3d.systems.RenderSystem;
import net.fabricmc.fabric.api.client.rendering.v1.HudRenderCallback;
import net.minecraft.block.SculkShriekerBlock;
import net.minecraft.block.entity.SculkShriekerWarningManager;
import net.minecraft.client.MinecraftClient;
import net.minecraft.client.font.TextRenderer;
import net.minecraft.client.gui.DrawContext;
import net.minecraft.client.gui.PlayerSkinDrawer;
import net.minecraft.client.gui.hud.InGameHud;
import net.minecraft.client.network.PlayerListEntry;
import net.minecraft.client.texture.PlayerSkinProvider;
import net.minecraft.entity.effect.StatusEffects;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.text.Style;
import net.minecraft.text.Text;
import net.minecraft.text.TextColor;
import net.minecraft.text.Texts;
import net.minecraft.util.Identifier;
import net.minecraft.util.math.MathHelper;

import java.util.*;

public class PlayerDamageOverlay implements HudRenderCallback {
    public static final int MAX_LIFETIME = 20 * 30;
    public static final int FADE_START_TIME = 20 * 25;
    public static final int FADE_LENGTH = MAX_LIFETIME - FADE_START_TIME;

    private final List<PlayerDamageInstance> instances = new ArrayList<>();

    private final Object o = new Object();

    int animationOffset = 0;
    float ticks = 0;

    @Override
    public void onHudRender(DrawContext drawContext, float tickDelta) {
        ticks += tickDelta;

        synchronized (o) {
            var client = MinecraftClient.getInstance();

            if (client == null)
                return;

            instances.sort(Comparator.comparingDouble(o -> o.lifetime));

            int i = 0;

            Set<PlayerDamageInstance> toRemove = new HashSet<>();

            if (animationOffset > 0)
                animationOffset--;

            for (PlayerDamageInstance instance : instances) {
                instance.addTime(tickDelta);

                if (instance.getHalfHeartCount() == 0) {
                    continue;
                }

                if (animationOffset == 0 || i == 0) {
                    this.renderLine(drawContext, instance, i++ * 11);
                } else {
                    this.renderLine(drawContext, instance, (i++ * 11) - animationOffset);
                }

                if (instance.lifetime >= MAX_LIFETIME)
                    toRemove.add(instance);
            }

            for (PlayerDamageInstance playerDamageInstance : toRemove) {
                instances.remove(playerDamageInstance);
            }
        }
    }

    public void pushInstance(PlayerDamageInstance i) {
        synchronized (o) {
            for (PlayerDamageInstance instance : this.instances) {
                if (instance.matches(i)) {
                    instance.updateHealth(i.health);
                    return;
                }
            }

            this.instances.add(i);
            animationOffset = 11;
        }
    }

    private float flip(float x) { return 1 - x; }
    private float easeIn(float t) { return t * t; }
    private float easeOut(float t) { return flip(easeIn(flip(t))); }
    private void renderLine(DrawContext drawContext, PlayerDamageInstance instance, int rawBaseY) {
        var client = MinecraftClient.getInstance();
        var window = client.getWindow();
        var world = Objects.requireNonNull(client.getCameraEntity()).getWorld();

        var hardcore = world.getLevelProperties().isHardcore();
        var blinking = instance.lifetime < 20 && Math.floor(ticks / 3.0F) % 2 == 1;

        float alpha = 1;

        if (instance.lifetime > FADE_START_TIME) {
            float progress = (instance.lifetime - FADE_START_TIME) / FADE_LENGTH;
            alpha = MathHelper.clampedLerp(1, 0, easeOut(progress));
        }

        int completelyEmptyHeartCount;

        if (instance.getHalfHeartCount() == 0) {
            completelyEmptyHeartCount = 0;
        } else if (instance.getHalfHeartCount() < 0) {
            completelyEmptyHeartCount = Math.floorDiv(-instance.getHalfHeartCount(), 2);
        } else {
            completelyEmptyHeartCount = (int) Math.ceil(((double) instance.getHalfHeartCount()) / 2.0F);
        }

        assert MinecraftClient.getInstance().player != null;
        int baseHeartRows = (int) (Math.ceil((Math.ceil(MinecraftClient.getInstance().player.getMaxHealth() / 2) + Math.ceil(MinecraftClient.getInstance().player.getAbsorptionAmount() / 2)) / 10));

        int baseX = (window.getScaledWidth() / 2) - 91;
        int baseY = window.getScaledHeight() - 40 - rawBaseY - (baseHeartRows * 10);

        for (int i = 0; i < completelyEmptyHeartCount; i++) {
            HeartType.CONTAINER.draw(drawContext, baseX + (i * 8), baseY, hardcore, false, blinking, i == (completelyEmptyHeartCount - 1), alpha);
        }

        if (instance.heartType.equals(HeartType.CONTAINER) && (-instance.getHalfHeartCount()) % 2 == 1) {
            HeartType.CONTAINER.draw(drawContext, baseX + (completelyEmptyHeartCount * 8), baseY, hardcore, true, blinking, true, alpha);
        }

        if (!instance.heartType.equals(HeartType.CONTAINER)) {
            if (Math.abs(instance.getHalfHeartCount()) % 2 == 1) {
                for (int i = 0; i < (Math.abs(completelyEmptyHeartCount) - 1); i++) {
                    instance.heartType.draw(drawContext, baseX + (i * 8), baseY, hardcore, false, blinking, false, alpha);
                }

                instance.heartType.draw(drawContext, baseX + ((Math.abs(completelyEmptyHeartCount) - 1) * 8), baseY, hardcore, true, blinking, true, alpha);
            } else {
                for (int i = 0; i < Math.abs(completelyEmptyHeartCount); i++) {
                    instance.heartType.draw(drawContext, baseX + (i * 8), baseY, hardcore, false, blinking, i == Math.abs(completelyEmptyHeartCount) - 1, alpha);
                }
            }
        }

        PlayerIconRenderer.draw(client.getSkinProvider().loadSkin(instance.profile), baseX - 10, baseY + 1, 8, true, alpha);

        Text t = Texts.join(List.of(new Text[] {
            instance.damageReason,
//            Optional.ofNullable(world.getPlayerByUuid(instance.profile.getId()))
//                .map(PlayerEntity::getDisplayName)
//                .orElse(Text.of(instance.profile.getName())),
        }), Text.of(" / "));

        var leftOffset = baseX - client.textRenderer.getWidth(t) - 12;

        int color = (((int) (alpha * 255)) << 24) | 0xFFFFFF;

        if ((alpha * 255) > 4) {
            drawContext.drawText(client.textRenderer, t, leftOffset, baseY + 1, color, true);
        }
    }
}
