package ly.hall.rose.healthsync;

import com.mojang.blaze3d.systems.RenderSystem;
import net.minecraft.client.gui.DrawContext;
import net.minecraft.client.gui.PlayerSkinDrawer;
import net.minecraft.client.render.*;
import net.minecraft.client.util.math.MatrixStack;
import net.minecraft.util.Identifier;
import org.joml.Matrix4f;

public class PlayerIconRenderer {
    public static void draw(Identifier texture, int x, int y, int size, boolean hatVisible, float opacity) {
        render(texture, x, y, size, size, 8.0f, 8, 8, 8, opacity);
        if (hatVisible) {
            drawHat(texture, x, y, size, opacity);
        }
    }

    private static void drawHat(Identifier texture, int x, int y, int size, float opacity) {
        render(texture, x, y, size, size, 40.0f, 8, 8, 8, opacity);
    }

    public static void render(Identifier texture, int x, int y, int width, int height, float u, float v, int regionWidth, int regionHeight, float opacity) {
        float u1 = u/64.0F;
        float v1 = v/64.0F;
        float u2 = u1 + (8.0F/64.0F);
        float v2 = v1 + (8.0F/64.0F);

        RenderSystem.setShaderTexture(0, texture);
        RenderSystem.setShader(GameRenderer::getPositionColorTexProgram);
        RenderSystem.enableBlend();
        Matrix4f matrix4f = (new MatrixStack()).peek().getPositionMatrix();
        BufferBuilder bufferBuilder = Tessellator.getInstance().getBuffer();
        bufferBuilder.begin(VertexFormat.DrawMode.QUADS, VertexFormats.POSITION_COLOR_TEXTURE);
        bufferBuilder.vertex(matrix4f, x, y, 0).color(1.0F, 1.0F, 1.0F, opacity).texture(u1, v1).next();
        bufferBuilder.vertex(matrix4f, x, y + height, 0).color(1.0F, 1.0F, 1.0F, opacity).texture(u1, v2).next();
        bufferBuilder.vertex(matrix4f, x + width, y + height, 0).color(1.0F, 1.0F, 1.0F, opacity).texture(u2, v2).next();
        bufferBuilder.vertex(matrix4f, x + width, y, 0).color(1.0F, 1.0F, 1.0F, opacity).texture(u2, v1).next();
        BufferRenderer.drawWithGlobalProgram(bufferBuilder.end());
        RenderSystem.disableBlend();
    }
}
