package ly.hall.rose.healthsync;

import net.fabricmc.fabric.api.entity.event.v1.ServerLivingEntityEvents;
import net.minecraft.entity.LivingEntity;
import net.minecraft.entity.damage.DamageSource;
import net.minecraft.entity.player.PlayerEntity;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.text.Text;

import java.util.HashSet;
import java.util.Set;

public class LivingEntityDamageHandler {
    private static Set<DamageCallback> callbackSet = new HashSet<>();
    private static Set<LivingEntity> entitiesTakingDamage = new HashSet<>();

    public static void initializeGeneric() {
        ServerLivingEntityEvents.ALLOW_DAMAGE.register((LivingEntity e, DamageSource s, float a) -> {
            var oldHealth = e.getHealth();
            var oldAbsorption = e.getAbsorptionAmount();

            entitiesTakingDamage.add(e);

            LivingEntityPostDamageEvent.once(e, () -> {
                var healthDelta = e.getHealth() - oldHealth;
                var absorptionDelta = e.getAbsorptionAmount() - oldAbsorption;

                entitiesTakingDamage.remove(e);

                LivingEntityDamageHandler.fire(e, s, healthDelta, absorptionDelta);
            });

            return true;
        });
    }

    public static void fire(LivingEntity e, DamageSource source, float healthAmount, float absorptionAmount) {
        if (!(e instanceof ServerPlayerEntity))
            return;

        if (source.isOf(ModDamageTypes.SYNCHRONIZATION_DAMAGE))
            return;

        callbackSet.forEach(cb -> cb.handle((ServerPlayerEntity) e, source, healthAmount, absorptionAmount));
    }

    public static void fireFromAbsorption(PlayerEntity e, float absorptionAmount) {
        if (entitiesTakingDamage.contains(e))
            return;

        fire(e, ModDamageTypes.of(e.getWorld(), ModDamageTypes.ABSORPTION_CHANGED), 0, absorptionAmount);
    }

    public static void handle(DamageCallback cb) {
        callbackSet.add(cb);
    }

    public static void unhandle(DamageCallback cb) {
        callbackSet.remove(cb);
    }
}
