package ly.hall.rose.healthsync;

import net.minecraft.entity.LivingEntity;
import net.minecraft.server.network.ServerPlayerEntity;

import java.util.HashSet;
import java.util.Optional;
import java.util.Set;

public class LivingEntityHealHandler {
    private static Set<HealCallback> callbackSet = new HashSet<>();
    private static LivingEntityHealReason appliedReason = null;
    public static Set<ServerPlayerEntity> loadingPlayers = new HashSet<>();

    public static void applyReason(LivingEntityHealReason reason) {
        appliedReason = reason;
    }

    private static LivingEntityHealReason consumeReason() {
        var reason = appliedReason;
        appliedReason = null;
        return reason;
    }

    public static void fire(LivingEntity e, float healthAmount, float absorptionAmount) {
        var reason = consumeReason();

        if (!(e instanceof ServerPlayerEntity))
            return;

        // handle setting health during construction
        if (((ServerPlayerEntity) e).getGameProfile() == null)
            return;

        if (reason == LivingEntityHealReason.Synchronization)
            return;

        if (loadingPlayers.contains(e))
            return;

        for (HealCallback healCallback : callbackSet) {
            healCallback.handle((ServerPlayerEntity) e, reason, healthAmount, absorptionAmount);
        }
    }

    public static void handle(HealCallback cb) {
        callbackSet.add(cb);
    }

    public static void unhandle(HealCallback cb) {
        callbackSet.remove(cb);
    }
}
