package ly.hall.rose.healthsync.mixin.heal;

import ly.hall.rose.healthsync.*;
import net.minecraft.entity.LivingEntity;
import net.minecraft.entity.effect.StatusEffectInstance;
import net.minecraft.entity.effect.StatusEffects;
import net.minecraft.server.network.ServerPlayerEntity;
import net.minecraft.util.math.MathHelper;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Overwrite;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfoReturnable;

@Mixin(LivingEntity.class)
public class LivingEntityMixin {
    @Inject(at = @At(value = "HEAD"), method = "setHealth")
    private void handleHealthUpdate(float targetHealth, CallbackInfo ci) {
        var currentHealth = ((LivingEntity)(Object)this).getHealth();
        var difference = Float.valueOf(MathHelper.clamp(targetHealth, 0.0f, ((LivingEntity)(Object)this).getMaxHealth())) - currentHealth;

        if (difference > 0) {
            LivingEntityHealHandler.fire((LivingEntity)(Object)this, difference, 0);
        }
    }

    @Inject(at = @At("HEAD"), method = "onStatusEffectRemoved")
    private void handleStatsuEffectRemoved(StatusEffectInstance effect, CallbackInfo ci) {
        var self = (LivingEntity) (Object) this;

        if (self instanceof ServerPlayerEntity) {
            if (effect.getEffectType().equals(StatusEffects.ABSORPTION)) {
                LivingEntityDamageHandler.fireFromAbsorption((ServerPlayerEntity) self, -self.getAbsorptionAmount());
            }
        }
    }
}
